MODULE	Crea ;

(*=============================================================================
   ASIGNATURA	: Programacion de Computadores 2
   OBJETIVO	: Generar un archivo binario resultado del analisis de xferlog
   AUTOR	: Andres Gomez Casanova
   FECHA	: Abril 26, 2002
=============================================================================*)


(*---------------------------------------------------------------------------*)
(* ENCABEZADO DE LOS MODULOS (Bibliotecas) *)
IMPORT	Tipo, Inte, Col1, Streams, Texts, Conv, Date ;


(*---------------------------------------------------------------------------*)
(* CONSTANTES *)
CONST
	RUTA	= "xferlog" ;		(* Ubicacion del origen *)
	LINEAS	= 100 ;			(* Lineas para punto en pantalla *)


(*---------------------------------------------------------------------------*)
(* TIPOS *)
TYPE
	Tdato	= RECORD ;
		usuario	: Col1.Tcola ;
		maquina	: Col1.Tcola ;
		archivo	: Col1.Tcola ;
		fecha	: Tipo.Tfecha ;
	END ;


(*---------------------------------------------------------------------------*)
(* DEFINICION E IMPLEMENTACION DE LOS SUBPROGRAMAS *)
(*-----------------------------------------------------------------------------
   PROCEDIMIENTO origen (SAL tipo : ENTERO)
-------------------------------------------------------------------------------
   OBJ		: Busca el archivo xferlog
   PRE		: VERDADERO
   POS		: tipo = 0 esta el archivo, 1 no est, salir del programa
-----------------------------------------------------------------------------*)
PROCEDURE origen (VAR tipo : INTEGER) ;
VAR
	origen	: Texts.Text ;	(* Archivo origen *)
BEGIN
(* Abre el archivo *)
	origen := Texts.Open (RUTA) ;

(* Si no pudo, es debido a que el archivo no se encuentra en RUTA*)
	IF (origen = NIL) THEN
		Inte.error (2) ;
		tipo := 1 ;
	ELSE
(* Si pudo, se comprueba que no este vacio *)
		IF Texts.Eof (origen) THEN
			Inte.error (5) ;
			tipo := 1 ;
		END ;
		Texts.Close (origen) ;
	END ;
END	origen ;


(*-----------------------------------------------------------------------------
   PROCEDIMIENTO fecha (ENT/SAL linea : Col1.Tcola, SAL datos : Tdato,
			SAL tipo : ENTERO)
-------------------------------------------------------------------------------
   OBJ		: Lee elementos de una cola de caracteres para analizarlos, y
		  segn el orden de estos se comparan unos patrones para
		  determinar la fecha.
   PRE		: linea = Col1.Tcola, lnea del archivo xferlog, donde sus
		  primeros caracteres son la fecha y hora.
   POS		: datos = Tdato, fecha analizada con su equivalente en
		  esta parte. linea ya no contiene la parte de la fecha, lista
		  para leer la mquina. tipo = entero, error en la lectura.
-----------------------------------------------------------------------------*)
PROCEDURE fecha (VAR linea : Col1.Tcola ; VAR datos : Tdato ;
		 VAR tipo : INTEGER) ;
VAR
	mes	: ARRAY 4 OF CHAR ;	(* Analisis del mes *)
	dia	: ARRAY 2 OF CHAR ;	(* Analisis del dia *)
	hor	: ARRAY 2 OF CHAR ;	(* Analisis de la hora *)
	min	: ARRAY 2 OF CHAR ; 	(* Analisis del minuto *)
	seg	: ARRAY 2 OF CHAR ; 	(* Analisis del segundo *)
	ano	: ARRAY 4 OF CHAR ;	(* Analisis del ano *)
BEGIN
(* Ajuste lectura *)
	Col1.desencolarCol (linea) ;
	Col1.desencolarCol (linea) ;
	Col1.desencolarCol (linea) ;
	Col1.desencolarCol (linea) ;

(* Analisis mes *)
	mes[0] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	mes[1] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	mes[2] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	mes[3] := 0X ;
	Date.meses (mes, datos.fecha, tipo) ;

(* Ajuste lectura *)
	Col1.desencolarCol (linea) ;

(* Analisis dia *)
	dia[0] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	dia[1] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	IF (dia[0] = " ") THEN
		dia[0] := "0" ;
	END ;
	datos.fecha.dia := Conv.IntVal (dia) ;

(* Ajuste lectura *)
	Col1.desencolarCol (linea) ;

(* Analisis hora *)
	hor[0] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	hor[1] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	datos.fecha.hor := Conv.IntVal (hor) ;

(* Ajuste lectura *)
	Col1.desencolarCol (linea) ;

(* Analisis minuto *)
	min[0] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	min[1] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	datos.fecha.min := Conv.IntVal (min) ;

(* Ajuste lectura *)
	Col1.desencolarCol (linea) ;

(* Analisis segundo *)
	seg[0] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	seg[1] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	datos.fecha.seg := Conv.IntVal (seg) ;

(* Ajuste lectura *)
	Col1.desencolarCol (linea) ;

(* Analisis agno *)
	ano[0] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	ano[1] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	ano[2] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	ano[3] := Col1.infoCol (linea) ;
	Col1.desencolarCol (linea) ;
	datos.fecha.ano := Conv.IntVal (ano) ;

(* Ajuste lectura *)
	Col1.desencolarCol (linea) ;
	WHILE (Col1.infoCol (linea) # " ") DO
		Col1.desencolarCol (linea) ;
	END ;
	Col1.desencolarCol (linea) ;
END	fecha ;


(*-----------------------------------------------------------------------------
   PROCEDIMIENTO maquina (ENT/SAL linea : Col1.Tcola, ENT/SAL datos : Tdato)
-------------------------------------------------------------------------------
   OBJ		: Lee elementos de una cola hasta encontrar un espacio y los
		  guarda en datos en donde corresponde a mquina.  Sigue
		  leyendo para eliminar elementos que no se van a analizar.
   PRE		: linea = Col1.Tcola, linea del archivo xferlog sin la parte
		  de la fecha, justo en la posicin para leer la mquina.
		  datos = Tdato
   POS		: datos = Tdato, con la maquina entre este.  linea sin la
		  parte de la mquina, lista para leer el archivo.
-----------------------------------------------------------------------------*)
PROCEDURE maquina (VAR linea : Col1.Tcola ; VAR datos : Tdato) ;
VAR
	car	: CHAR ;
BEGIN
(* Crea cola para leer maquina *)
	Col1.crearCol (datos.maquina) ;

(* Recorre la linea de xferlog hasta encotrar un espacio y la guarda en dato *)
	car := Col1.infoCol (linea) ;
	WHILE (car # " ") DO
		Col1.encolarCol (datos.maquina, car) ;
		Col1.desencolarCol (linea) ;
		car := Col1.infoCol (linea) ;
	END ;

(* Se ubica para leer el archivo *)
	Col1.desencolarCol (linea) ;
	car := Col1.infoCol (linea) ;
	WHILE (car # " ") DO
		car := Col1.infoCol (linea) ;
		Col1.desencolarCol (linea) ;
	END ;
END	maquina ;


(*-----------------------------------------------------------------------------
   PROCEDIMIENTO archivo (ENT/SAL linea : Col1.Tcola, ENT/SAL datos : Tdato,
			  SAL tipo : ENTERO)
-------------------------------------------------------------------------------
   OBJ		: Lee elementos de una cola hasta encontrar un espacio y los
		  guarda en datos el cual corresponde a archivo.  Sigue leyendo
		  para eliminar elementos que no se van a analizar.
   PRE		: linea = Col1.Tcola, linea del archivo xferlog sin la parte
		  de la fecha ni la mquina, justo en la posicin para leer el
		  archivo.  datos =.Tdato
   POS		: datos = Tdato, con el archivo entre este.  linea sin la parte
		  del archivo, lista para leer el usuario. tipo = error.
-----------------------------------------------------------------------------*)
PROCEDURE archivo (VAR linea : Col1.Tcola ; VAR datos : Tdato) ;
VAR
	i	: INTEGER ;	(* Contador de posiciones a eliminar *)
	car	: CHAR ;	(* Caracter de lectura *)
BEGIN
(* Crea cola para archivo *)
	Col1.crearCol (datos.archivo) ;

(* Lee la linea del xferlog hasta encontrar un espacio y la mete en dato *)
	car := Col1.infoCol (linea) ;
	WHILE (car # " ") DO
		Col1.encolarCol (datos.archivo, car) ;
		Col1.desencolarCol (linea) ;
		car := Col1.infoCol (linea) ;
	END ;

(* Se ubica para leer el usuario *)
	i := 0 ;
	REPEAT
		Col1.desencolarCol (linea) ;
		INC (i) ;
	UNTIL (i = 9) ;
END	archivo ;


(*-----------------------------------------------------------------------------
   PROCEDIMIENTO usuario (ENT/SAL linea : Col1.Tcola, ENT/SAL datos : Tdato)
-------------------------------------------------------------------------------
   OBJ		: Lee elementos de una cola hasta encontrar un espacio y los
		  guarda en datos, el cual corresponde a usuario.  Sigue
		  leyendo para eliminar elementos que no se van a analizar.
   PRE		: linea = Col1.Tcola, linea del archivo xferlog sin la parte
		  de la fecha, ni la mquina, ni el archivo, justo en la
		  posicin para leer el usuario.  datos = Tdato
   POS		: datos = Tdato, con el nombre de usuario entre este.
		  linea vacia.
-----------------------------------------------------------------------------*)
PROCEDURE usuario (VAR linea : Col1.Tcola ; VAR datos : Tdato ;
		   VAR tipo : INTEGER) ;
VAR
	car	: CHAR ;	(* Caracter de lectura *)
	i	: INTEGER ;	(* Longitud usuario *)
BEGIN
(* Crea una cola para leer el usuario *)
	Col1.crearCol (datos.usuario);

(* Lee la linea del xferlog hasya un espacio y la guarda en el tipo *)
	car := Col1.infoCol (linea) ;
	i := 0 ;
	WHILE (car # " ") DO
		Col1.encolarCol (datos.usuario, car) ;
		Col1.desencolarCol (linea) ;
		car := Col1.infoCol (linea) ;
		INC (i) ;
	END ;

(* Error en la longitud del nombre de usuario, maximo 8 *)
	IF (i > 8) THEN
		tipo := 1 ;
	END ;

(* Vacia la cola de linea *)
	WHILE ~Col1.esVaciaCol (linea) DO
		Col1.desencolarCol (linea) ;
	END ;
END	usuario ;


(*-----------------------------------------------------------------------------
   PROCEDIMIENTO escritura (ENT/SAL datos : Tdato, ENT temp : Col1.Tcola,
			    ENT destino : Streams.Stream,  
-------------------------------------------------------------------------------
   OBJ		: Escribe los datos en el binario
   PRE		: datos analizado, destino abierto
   POS		: destino con los datos binarios
-----------------------------------------------------------------------------*)
PROCEDURE escritura (VAR datos : Tdato ; temp : Col1.Tcola ;
		     destino : Streams.Stream) ;
VAR
	i	: INTEGER ;		(* Posicion a guardar *)
BEGIN
(* Lee la cantidad de caracteres de usuario y lo escribe en binario *)
	i := 0 ;
	WHILE ~Col1.esVaciaCol (datos.usuario) DO
		Col1.encolarCol (temp, Col1.infoCol (datos.usuario)) ;
		Col1.desencolarCol (datos.usuario) ;
		INC (i) ;
	END ;
	Streams.WriteInt (destino, i) ;

(* Escribe el usuario en el binario *)
	WHILE ~Col1.esVaciaCol (temp) DO
		Streams.WriteChar (destino, Col1.infoCol (temp)) ;
		Col1.desencolarCol (temp) ;
	END;

(* Lee la cantidad de caracteres de maquina y lo escribe en binario *)
	i := 0 ;
	WHILE ~Col1.esVaciaCol (datos.maquina) DO
		Col1.encolarCol (temp, Col1.infoCol (datos.maquina)) ;
		Col1.desencolarCol (datos.maquina) ;
		INC (i) ;
	END ;
	Streams.WriteInt (destino, i) ;

(* Escribe la maquina en el binario *)
	WHILE ~Col1.esVaciaCol (temp) DO
		Streams.WriteChar (destino, Col1.infoCol (temp)) ;
		Col1.desencolarCol (temp) ;
	END;

(* Lee la cantidad de caracteres de archivo y lo escribe en binario *)
	i := 0 ;
	WHILE ~Col1.esVaciaCol (datos.archivo) DO
		Col1.encolarCol (temp, Col1.infoCol (datos.archivo)) ;
		Col1.desencolarCol (datos.archivo) ;
		INC (i) ;
	END ;
	Streams.WriteInt (destino, i) ;

(* Escribe el archivo en el binario *)
	WHILE ~Col1.esVaciaCol (temp) DO
		Streams.WriteChar (destino, Col1.infoCol (temp)) ;
		Col1.desencolarCol (temp) ;
	END;

(* Escribe la fecha en el binario *)
	Streams.WriteInt (destino, datos.fecha.ano) ;
	Streams.WriteInt (destino, datos.fecha.mes) ;
	Streams.WriteInt (destino, datos.fecha.dia) ;
	Streams.WriteInt (destino, datos.fecha.hor) ;
	Streams.WriteInt (destino, datos.fecha.min) ;
	Streams.WriteInt (destino, datos.fecha.seg) ;
END	escritura ;


(*-----------------------------------------------------------------------------
   PROCEDIMIENTO lecturaArchivo (ENT destino : Tipo.Tvector ; SAL tipo : 
				ENTERO)
-------------------------------------------------------------------------------
   OBJ		: Lee el archivo origen, lo analiza y guarda en el archivo
		  destino los datos relevantes.
   PRE		: destino = Tipo.Tvector, donde es un nombre de archivo con
		  el que se puede trabajar y ya est creado
   POS		: tipo = ENTERO, error en la lectura
-----------------------------------------------------------------------------*)
PROCEDURE lecturaArchivo (nombre : Tipo.Tvector ; VAR tipo : INTEGER) ;
VAR
	origen	: Texts.Text ;		(* Archivo a analizar *)
	destino	: Streams.Stream ;	(* Archivo binario con analisis *)
	cola	: Col1.Tcola ;		(* Cola de lectura de linea *)
	temp	: Col1.Tcola ;		(* Cola de longitud de caracteres *)
	car	: CHAR ;		(* Caracter de lectura de linea *)
	datos	: Tdato ;		(* Analisis de cada linea *)
	j	: INTEGER ;		(* Puntos para interfaz *)
BEGIN
(* Numero de lineas analizadas *)
	j := 0 ;

(* Abre archivo origen y destino *)
	origen := Texts.Open (RUTA) ;
	destino := Streams.Open (nombre) ;

(* Cola para leer la linea por caracteres *)		
	Col1.crearCol (cola) ;
	Col1.crearCol (temp) ;

(* Lee el archivo xferlog hasta llegar al final *)
	WHILE ~Texts.Eof (origen) & (tipo = 0) DO

(* Lee una linea del xferlog hasta llegar al salto de linea *)
		Texts.ReadChar (origen, car) ;

(* Comprueba que no haya lineas en blanco en medio del archivo *)
		IF (car = " ") OR (car = 0AX) THEN
			Inte.error (3) ;
			tipo := 1 ;
		ELSE
(* Guarda lo que lee en la cola *)
			WHILE (car # 0AX) DO
				Col1.encolarCol (cola, car) ;
				Texts.ReadChar (origen, car) ;
			END ;

(* Analiza la linea por partes *)
			fecha (cola, datos, tipo) ;
			maquina (cola, datos) ;
			archivo (cola, datos) ;
			usuario (cola, datos, tipo) ;

(* Escribe los datos en el archivo binario *)
			escritura (datos, temp, destino) ;

(* Si se han leido 100 lineas muestra un punto en pantalla *)
			INC (j) ;
			IF (j MOD LINEAS = 0) THEN
				Inte.punto () ;
			END ;
		END ;
	END ;
	Texts.Close (origen) ;
	Streams.Close (destino) ;		
END	lecturaArchivo ;


(*-----------------------------------------------------------------------------
   PROCEDIMIENTO generar (ENT nombre : Tipo.Tvector, SAL tipo : ENTERO)
-------------------------------------------------------------------------------
   OBJ		: Ejecuta las operaciones correspondientes para generar el
		  archivo binario
   PRE		: nombre = Tipo.Tvector que es el nombre de un archivo creado
		  vaco.
   POS		: tipo = ENTERO, en donde se retorna el error generado, 0 si se
		  puede continuar ejecutando.
-----------------------------------------------------------------------------*)
PROCEDURE generar* (nombre : Tipo.Tvector ; VAR tipo : INTEGER) ;

BEGIN
(* Comprueba si el archivo xferlog esta *)
	origen	(tipo) ;

(* Lee el archivo xferlog y guarda algunos datos en el binario *)
	IF (tipo = 0) THEN
		Inte.analizando (RUTA) ;
	END ;
	IF (tipo = 0) THEN
		lecturaArchivo (nombre, tipo) ;
	END ;
END	generar ;


END	Crea.
